@@ -2096,6 +2096,27 @@ static uint32_t _tpm20_context_flush(uint32_t locality,
     
 }
 
+static bool tpm20_context_flush(struct tpm_if *ti, u32 locality, TPM_HANDLE handle)
+{
+    tpm_flushcontext_in in;
+    u32 ret;
+    
+    if ( ti == NULL || locality >= TPM_NR_LOCALITIES )
+        return false;
+    if ( handle == 0 )
+        return false;
+    in.flushHandle = handle;
+        ret = _tpm20_context_flush(locality, &in);
+    if ( ret != TPM_RC_SUCCESS ) {
+        printk(TBOOT_WARN"TPM: tpm2 context flush returned , return value = %08X\n", ret);
+        ti->error = ret;
+        return false;
+    }
+    else 
+        printk(TBOOT_WARN"TPM: tpm2 context flush successful, return value = %08X\n", ret);
+    return true;
+}	
+
 
 TPM_CMD_SESSION_DATA_IN pw_session;
 static void create_pw_session(TPM_CMD_SESSION_DATA_IN *ses)
@@ -2513,6 +2534,12 @@ static bool tpm20_unseal(struct tpm_if *ti, uint32_t locality,
     *secret_size = unseal_out.data.t.size;
     tb_memcpy(secret, &(unseal_out.data.t.buffer[0]), *secret_size);
 
+    if ( !tpm20_context_flush(ti, locality, load_out.obj_handle) ) {
+        printk(TBOOT_WARN"TPM: Failed to flush context\n");
+        ti->error = ret;
+        return false;
+    }
+
     return true;
 }
 
@@ -2693,27 +2720,6 @@ static bool tpm20_context_load(struct tpm_if *ti, u32 locality, void  *context_s
     return true;
 }	
 
-static bool tpm20_context_flush(struct tpm_if *ti, u32 locality, TPM_HANDLE handle)
-{
-    tpm_flushcontext_in in;
-    u32 ret;
-    
-    if ( ti == NULL || locality >= TPM_NR_LOCALITIES )
-        return false;
-    if ( handle == 0 )
-        return false;
-    in.flushHandle = handle;
-        ret = _tpm20_context_flush(locality, &in);
-    if ( ret != TPM_RC_SUCCESS ) {
-        printk(TBOOT_WARN"TPM: tpm2 context flush returned , return value = %08X\n", ret);
-        ti->error = ret;
-        return false;
-    }
-    else 
-        printk(TBOOT_WARN"TPM: tpm2 context flush successful, return value = %08X\n", ret);
-    return true;
-}	
-
 static bool tpm20_init(struct tpm_if *ti)
 {
     u32 ret;
